<?php
/*
Plugin Name: Wassenger Order Notifications
Plugin URI: https://wassenger.in/
Description: Sends custom WhatsApp messages on WooCommerce order status change, with dynamic customer phone numbers, selectable order statuses, and custom messages for each status.
Version: 1.0
Author: Wassenger
Author URI: https://wassenger.in/
License: GPL2
*/

// Prevent direct access
if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

// Register the plugin settings
function wc_wa_order_notifications_settings() {
    add_option( 'wc_wa_token', '' );
    add_option( 'wc_wa_instance_id', '' );
    add_option( 'wc_wa_trigger_statuses', array() );  // Store selected statuses
    add_option( 'wc_wa_custom_messages', array() );  // Store custom messages

    register_setting( 'wc_wa_options_group', 'wc_wa_token' );
    register_setting( 'wc_wa_options_group', 'wc_wa_instance_id' );
    register_setting( 'wc_wa_options_group', 'wc_wa_trigger_statuses' );  // Store selected statuses
    register_setting( 'wc_wa_options_group', 'wc_wa_custom_messages' );  // Store custom messages
}
add_action( 'admin_init', 'wc_wa_order_notifications_settings' );

// Add settings page
function wc_wa_order_notifications_menu() {
    add_submenu_page(
        'woocommerce',
        'Wassenger Order Notification Settings',
        'Wassenger',
        'manage_options',
        'wc_wa_order_notifications',
        'wc_wa_order_notifications_page'
    );
}
add_action( 'admin_menu', 'wc_wa_order_notifications_menu' );

// Settings page HTML
function wc_wa_order_notifications_page() {
    // WooCommerce order statuses
    $statuses = array(
        'pending' => 'Pending payment',
        'on-hold' => 'On Hold',
        'processing' => 'Processing',
        'completed' => 'Completed',
        'failed' => 'Failed',
        'draft' => 'Draft',
        'cancelled' => 'Cancelled',
        'refunded' => 'Refunded'
    );

    // Get the selected order statuses from the settings
    $selected_statuses = get_option( 'wc_wa_trigger_statuses', array() );
    $custom_messages = get_option( 'wc_wa_custom_messages', array() );

    // Ensure selected statuses and custom messages are arrays
    if ( ! is_array( $selected_statuses ) ) {
        $selected_statuses = array();
    }

    if ( ! is_array( $custom_messages ) ) {
        $custom_messages = array();
    }

    ?>
    <div class="wrap">
        <h2>Wassenger Order Notification Settings</h2>
        <form method="post" action="options.php">
            <?php settings_fields( 'wc_wa_options_group' ); ?>
            <table class="form-table">
                <tr valign="top">
                    <th scope="row">API Token</th>
                    <td>
                        <input type="text" name="wc_wa_token" value="<?php echo esc_attr(get_option('wc_wa_token')); ?>" class="regular-text" />
                        <p class="description">
                            <a href="https://app.wassenger.in/user?page=api-access" target="_blank">Get your API Token here</a>
                        </p>
                    </td>
                </tr>

                <tr valign="top">
                    <th scope="row">Instance ID</th>
                    <td>
                        <input type="text" name="wc_wa_instance_id" value="<?php echo esc_attr(get_option('wc_wa_instance_id')); ?>" class="regular-text" />
                        <p class="description">
                            <a href="https://app.wassenger.in/user?page=api-access" target="_blank">Get your Instance ID here</a>
                        </p>
                    </td>
                </tr>

                <tr valign="top">
                    <th scope="row">Select Order Statuses to Trigger Notifications</th>
                    <td>
                        <?php foreach ( $statuses as $status_key => $status_name ): ?>
                            <label>
                                <input type="checkbox" name="wc_wa_trigger_statuses[]" value="<?php echo $status_key; ?>" <?php echo in_array( $status_key, (array) $selected_statuses ) ? 'checked' : ''; ?>>
                                <?php echo $status_name; ?>
                            </label><br>
                        <?php endforeach; ?>
                        <p class="description">Select the order statuses that will trigger Wassenger notifications.</p>
                    </td>
                </tr>

                <?php
                // Loop over selected statuses and display the custom message textarea
                if ( ! empty( $selected_statuses ) ) {
                    foreach ( $selected_statuses as $status_key ) :
                        ?>
                        <tr valign="top">
                            <th scope="row">Custom Message for "<?php echo isset($statuses[$status_key]) ? $statuses[$status_key] : ''; ?>"</th>
                            <td>
                                <textarea name="wc_wa_custom_messages[<?php echo $status_key; ?>]" rows="5" class="large-text"><?php echo isset( $custom_messages[$status_key] ) ? esc_textarea( $custom_messages[$status_key] ) : ''; ?></textarea>
                                <p class="description">Define the custom message for the "<?php echo isset($statuses[$status_key]) ? $statuses[$status_key] : ''; ?>" order status. Use {first_name}, {last_name}, {order_id}, and {order_status} to dynamically insert order details.</p>
                            </td>
                        </tr>
                        <?php
                    endforeach;
                }
                ?>

            </table>
            <?php submit_button(); ?>
        </form>
    </div>
    <?php
}

// Hook into WooCommerce order creation to retrieve the customer phone number
function wc_wa_order_customer_phone( $order_id ) {
    // Get WooCommerce order object
    $order = wc_get_order( $order_id );

    // Get the customer's phone number (we use billing phone)
    $phone_number = $order->get_billing_phone();

    return $phone_number;
}

// Send Wassenger notification on order status change
function wc_wa_send_notification_on_order_status_change( $order_id, $old_status, $new_status ) {
    // Get WooCommerce order object
    $order = wc_get_order( $order_id );

    // Fetch the user-defined settings
    $token = get_option( 'wc_wa_token' );
    $instance_id = get_option( 'wc_wa_instance_id' );
    
    // Fetch selected order statuses that trigger notifications
    $selected_statuses = get_option( 'wc_wa_trigger_statuses', array() );

    // Ensure $selected_statuses is always an array
    if ( ! is_array( $selected_statuses ) ) {
        $selected_statuses = array(); // Default to an empty array
    }

    // Check if the current status is in the selected statuses
    if ( ! in_array( $new_status, $selected_statuses ) ) {
        return; // If the status is not selected, don't send the message
    }

    // Get the custom message for the selected status
    $custom_messages = get_option( 'wc_wa_custom_messages', array() );
    $message = isset( $custom_messages[$new_status] ) ? $custom_messages[$new_status] : '';

    // Check if the message exists for the current order status
    if ( empty( $message ) ) {
        return; // If no custom message is set for this status, do not send the message
    }

    // Get customer details
    $first_name = $order->get_billing_first_name();
    $last_name = $order->get_billing_last_name();
    $jid = wc_wa_order_customer_phone( $order_id );

    // Ensure all necessary settings are provided
    if ( ! $token || ! $instance_id || ! $jid || ! $message ) {
        return;
    }

    // Replace placeholders with dynamic data
    $message = str_replace( '{order_id}', $order->get_id(), $message );
    $message = str_replace( '{order_status}', $new_status, $message );
    $message = str_replace( '{first_name}', $first_name, $message );
    $message = str_replace( '{last_name}', $last_name, $message );
    $message = urlencode( $message ); // URL encode the message

    // Send the GET request
    $url = "https://www.app.wassenger.in/api/v1/send-text?token={$token}&instance_id={$instance_id}&jid={$jid}@s.whatsapp.net&msg={$message}";
    
    // Send the request using wp_remote_get
    $response = wp_remote_get( $url );

    // Check for success or failure in response
    if ( is_wp_error( $response ) ) {
        $error_message = $response->get_error_message();
        error_log( "Wassenger message failed: {$error_message}" );
    } else {
        $body = wp_remote_retrieve_body( $response );
        $data = json_decode( $body, true );
        
        if ( isset( $data['success'] ) && $data['success'] ) {
            // Message sent successfully
            error_log( "Wassenger message sent successfully for Order #{$order->get_id()}" );
        } else {
            // Failed to send message
            error_log( "Failed to send Wassenger message: " . $data['message'] );
        }
    }
}
add_action( 'woocommerce_order_status_changed', 'wc_wa_send_notification_on_order_status_change', 10, 3 );
?>
